# -*- coding: UTF-8 -*-
"""
Name: one_pass.py
Porpose: FFmpeg long processing task for one pass processing
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2023 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Dec.02.2022
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
from threading import Thread
import time
import itertools
import subprocess
import platform
import wx
from pubsub import pub
from videomass.vdms_utils.utils import Popen
from videomass.vdms_io.make_filelog import logwrite
if not platform.system() == 'Windows':
    import shlex


class OnePass(Thread):
    """
    This class represents a separate thread for running processes,
    which need to read the stdout/stderr in real time.

    capturing output in real-time (Windows, Unix):
    https://stackoverflow.com/questions/1388753/how-to-get-output-
    from-subprocess-popen-proc-stdout-readline-blocks-no-dat?rq=1
    """
    get = wx.GetApp()  # get videomass wx.App attributes
    appdata = get.appset
    NOT_EXIST_MSG = _("Is 'ffmpeg' installed on your system?")
    # ---------------------------------------------------------------

    def __init__(self, logname, duration, timeseq, *args):
        """
        Called from `long_processing_task.topic_thread`.
        Also see `main_frame.switch_to_processing`.
        """
        self.stop_work_thread = False  # process terminate
        self.input_flist = args[1]  # list of infile (items)
        self.command = args[4]  # comand set on single pass
        self.output_flist = args[3]  # output path
        self.duration = duration  # duration list
        self.volume = args[7]  # (lista norm.)se non richiesto rimane None
        self.count = 0  # count first for loop
        self.countmax = len(args[1])  # length file list
        self.logname = logname  # title name of file log
        self.time_seq = timeseq  # a time segment

        Thread.__init__(self)

        self.start()  # start the thread

    def run(self):
        """
        Thread started.
        """
        filedone = []
        for (infile,
             outfile,
             volume,
             duration) in itertools.zip_longest(self.input_flist,
                                                self.output_flist,
                                                self.volume,
                                                self.duration,
                                                fillvalue='',
                                                ):

            cmd = (f'"{OnePass.appdata["ffmpeg_cmd"]}" {self.time_seq} '
                   f'{OnePass.appdata["ffmpegloglev"]} '
                   f'{OnePass.appdata["ffmpeg+params"]} -i '
                   f'"{infile}" {self.command} {volume} '
                   f'{OnePass.appdata["ffthreads"]} -y "{outfile}"')

            self.count += 1
            count = f'File {self.count}/{self.countmax}'
            com = (f'{count}\nSource: "{infile}"\nDestination: "{outfile}"'
                   f'\n\n[COMMAND]:\n{cmd}')

            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=count,
                         fsource=f'Source:  "{infile}"',
                         destination=f'Destination:  "{outfile}"',
                         duration=duration,
                         end='',
                         )
            logwrite(com, '', self.logname)  # write n/n + command only

            if not platform.system() == 'Windows':
                cmd = shlex.split(cmd)

            try:
                with Popen(cmd,
                           stderr=subprocess.PIPE,
                           bufsize=1,
                           universal_newlines=True,
                           encoding='utf8',
                           ) as proc:
                    for line in proc.stderr:
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line,
                                     duration=duration,
                                     status=0,
                                     )
                        if self.stop_work_thread:
                            proc.terminate()
                            break  # break second 'for' loop

                    if proc.wait():  # error
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output='',
                                     duration=duration,
                                     status=proc.wait(),
                                     )
                        logwrite('',
                                 f"Exit status: {proc.wait()}",
                                 self.logname,
                                 )  # append exit error number
                    else:  # ok
                        filedone.append(infile)
                        wx.CallAfter(pub.sendMessage,
                                     "COUNT_EVT",
                                     count='',
                                     fsource='',
                                     destination='',
                                     duration=duration,
                                     end='Done'
                                     )
            except (OSError, FileNotFoundError) as err:
                excepterr = f"{err}\n  {OnePass.NOT_EXIST_MSG}"
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count=excepterr,
                             fsource='',
                             destination='',
                             duration=0,
                             end='error',
                             )
                break

            if self.stop_work_thread:
                proc.terminate()
                break  # break second 'for' loop

        time.sleep(.5)
        wx.CallAfter(pub.sendMessage, "END_EVT", msg=filedone)
    # --------------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True



# -*- coding: UTF-8 -*-
"""
FileName: libaom.py
Porpose: Contains AV1 functionality for A/V Conversions
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2023 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.04.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx
import wx.lib.scrolledpanel as scrolled


class AV1Pan(scrolled.ScrolledPanel):
    """
    This scroll panel implements controls for extra options
    of the `av1-libaom` encoder.
    """
    def __init__(self, parent, opt, osplat):
        """
        This is a child of `AV_Conv` class-panel (parent) and the `opt`
        attribute is a dict owned by that class.
        """
        self.parent = parent
        self.opt = opt
        scrolled.ScrolledPanel.__init__(self, parent, -1,
                                        size=(300, 700),
                                        style=wx.TAB_TRAVERSAL
                                        | wx.BORDER_NONE,
                                        name="AV1 scrollpanel",
                                        )
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((10, 10), 0)
        infomsg = "AV1 - libaom-av1"
        lbl_info = wx.StaticText(self, wx.ID_ANY, label=infomsg)
        sizer.Add(lbl_info, 0, wx.ALL | wx.CENTER, 5)
        if osplat == 'Darwin':
            lbl_info.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        else:
            lbl_info.SetFont(wx.Font(9, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        sizer.Add((10, 10), 0)
        self.rdb_usage = wx.RadioBox(self, wx.ID_ANY,
                                     (_("Quality and Compression")),
                                     choices=[("good"), ("realtime"),
                                              ],
                                     majorDimension=0,
                                     style=wx.RA_SPECIFY_ROWS,
                                     )
        sizer.Add(self.rdb_usage, 0, wx.TOP | wx.CENTRE, 5)
        lab_cpu = wx.StaticText(self, wx.ID_ANY, (_("Quality and Speed:")))
        sizer.Add(lab_cpu, 0, wx.TOP | wx.ALIGN_CENTER_HORIZONTAL, 5)
        self.spin_cpu = wx.SpinCtrl(self, wx.ID_ANY,
                                    "1", min=0,
                                    max=8, size=(-1, -1),
                                    style=wx.TE_PROCESS_ENTER,
                                    )
        sizer.Add(self.spin_cpu, 0, wx.TOP | wx.CENTRE, 5)

        lab_gop = wx.StaticText(self, wx.ID_ANY, ("Group of picture (GOP):"))
        sizer.Add(lab_gop, 0, wx.TOP | wx.ALIGN_CENTER_HORIZONTAL, 5)
        self.spin_gop = wx.SpinCtrl(self, wx.ID_ANY,
                                    "250", min=-1,
                                    max=1000, size=(-1, -1),
                                    style=wx.TE_PROCESS_ENTER,
                                    )
        sizer.Add(self.spin_gop, 0, wx.TOP | wx.CENTRE, 5)

        self.SetSizer(sizer)  # set panel
        self.SetAutoLayout(1)
        self.SetupScrolling()

        tip = (_('"good" is the default and recommended for most '
                 'applications; "realtime" is recommended for live/fast '
                 'encoding (live streaming, video conferencing, etc)'))
        self.rdb_usage.SetToolTip(tip)
        tip = (_('"cpu-used" sets how efficient the compression will be. '
                 'Default is 1. Lower values mean slower encoding with '
                 'better quality, and vice-versa. Valid values are from 0 '
                 'to 8 inclusive.'))
        self.spin_cpu.SetToolTip(tip)

        tip = (_('The "GOP" option can be used to set the maximum keyframe '
                 'interval. Anything up to 10 seconds is considered '
                 'reasonable for most content, so for 30 frames per second '
                 'content one would set to 300, for 60 fps content set '
                 'to 600. For "intra-only" output, set to 0. Set to -1 to '
                 'disable this control.'))
        self.spin_gop.SetToolTip(tip)

        self.Bind(wx.EVT_RADIOBOX, self.on_usage, self.rdb_usage)
        self.Bind(wx.EVT_SPINCTRL, self.on_cpu_used, self.spin_cpu)
        self.Bind(wx.EVT_SPINCTRL, self.on_gop, self.spin_gop)
    # ------------------------------------------------------------------#

    def default(self):
        """
        Set to default
        """
        self.spin_gop.SetValue(250)
        self.opt["GOP"] = '-g 250 -keyint_min 250'
        self.opt["RowMthreading"] = '-row-mt 1 -tiles 2x2'
        self.rdb_usage.SetSelection(0)
        self.opt["Usage"] = '-usage good'
        self.spin_cpu.SetValue(1)
        self.opt["CpuUsed"] = '-cpu-used 1'

        self.opt["Preset"] = ''
        self.opt["Profile"] = ''
        self.opt["Level"] = ''
        self.opt["Tune"] = ''
        self.opt["Deadline"] = ''

    # ------------------------------------------------------------------#

    def on_usage(self, event):
        """
        Set d-eadline
        """
        val = self.rdb_usage.GetStringSelection()
        self.opt["Usage"] = f'-usage {val}'
    # ------------------------------------------------------------------#

    def on_cpu_used(self, event):
        """
        Set cpu-used
        """
        self.opt["CpuUsed"] = f'-cpu-used {self.spin_cpu.GetValue()}'
    # ------------------------------------------------------------------#

    def on_gop(self, event):
        """
        Set group of pictures (GOP)
        """
        val = self.spin_gop.GetValue()
        if val == -1:
            self.opt["GOP"] = ''
        elif val == 0:
            self.opt["GOP"] = f'-g {val}'
        else:
            self.opt["GOP"] = f'-g {val} -keyint_min {val}'
    # ------------------------------------------------------------------#



# -*- coding: UTF-8 -*-
"""
FileName: webm.py
Porpose: Contains vp8/vp9 functionality for A/V Conversions
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2023 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.03.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx
import wx.lib.scrolledpanel as scrolled


class WebMPan(scrolled.ScrolledPanel):
    """
    This scroll panel implements controls for extra options
    of the `vp8/vp9` encoders.
    """

    def __init__(self, parent, opt, osplat):
        """
        This is a child of `AV_Conv` class-panel (parent) and the `opt`
        attribute is a dict owned by that class.
        """
        self.parent = parent
        self.opt = opt
        scrolled.ScrolledPanel.__init__(self, parent, -1,
                                        size=(300, 700),
                                        style=wx.TAB_TRAVERSAL
                                        | wx.BORDER_NONE,
                                        name="WEBM scrolledpanel",
                                        )
        sizerbase = wx.BoxSizer(wx.VERTICAL)
        sizerbase.Add((10, 10), 0)
        infomsg = "WebM"
        self.lbl_info = wx.StaticText(self, wx.ID_ANY, label=infomsg)
        sizerbase.Add(self.lbl_info, 0, wx.ALL | wx.CENTER, 5)
        if osplat == 'Darwin':
            self.lbl_info.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        else:
            self.lbl_info.SetFont(wx.Font(9, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        sizerbase.Add((10, 10), 0)
        self.rdb_deadline = wx.RadioBox(self, wx.ID_ANY,
                                        (_("Quality and Compression")),
                                        choices=[("best"),
                                                 ("good"),
                                                 ("realtime"),
                                                 ],
                                        majorDimension=0,
                                        style=wx.RA_SPECIFY_ROWS,
                                        )
        sizerbase.Add(self.rdb_deadline, 0, wx.TOP | wx.CENTRE, 5)
        lab_cpu = wx.StaticText(self, wx.ID_ANY, (_("Quality and Speed:")))
        sizerbase.Add(lab_cpu, 0, wx.TOP | wx.ALIGN_CENTER_HORIZONTAL, 5)
        self.spin_cpu = wx.SpinCtrl(self, wx.ID_ANY, "1", min=-8,
                                    max=8, size=(-1, -1),
                                    style=wx.TE_PROCESS_ENTER,
                                    )
        sizerbase.Add(self.spin_cpu, 0, wx.TOP | wx.CENTRE, 5)
        self.ckbx_rowmt = wx.CheckBox(self,
                                      wx.ID_ANY,
                                      ('Row based multithreading'),
                                      )
        sizerbase.Add(self.ckbx_rowmt, 0, wx.TOP | wx.CENTRE, 5)

        self.SetSizer(sizerbase)  # set panel
        self.SetAutoLayout(1)
        self.SetupScrolling()

        tip = (_('"good" is the default and recommended for most '
                 'applications; "best" is recommended if you have lots of '
                 'time and want the best compression efficiency; "realtime" '
                 'is recommended for live/fast encoding'))
        self.rdb_deadline.SetToolTip(tip)
        tip = (_('"cpu-used" sets how efficient the compression will be. '
                 'The meaning depends on the Quality and Compression '
                 'mode above.'))
        self.spin_cpu.SetToolTip(tip)

        self.Bind(wx.EVT_RADIOBOX, self.on_deadline, self.rdb_deadline)
        self.Bind(wx.EVT_SPINCTRL, self.on_cpu_used, self.spin_cpu)
        self.Bind(wx.EVT_CHECKBOX, self.on_row_mt, self.ckbx_rowmt)
    # ------------------------------------------------------------------#

    def default(self):
        """
        Set to default
        """
        if self.opt["VideoCodec"] == "-c:v libvpx":
            self.ckbx_rowmt.SetValue(False)
            self.ckbx_rowmt.Disable()
            self.opt["RowMthreading"] = ''
            self.lbl_info.SetLabel("VP8 - libvpx")
        else:
            self.ckbx_rowmt.SetValue(True)
            self.ckbx_rowmt.Enable()
            self.opt["RowMthreading"] = '-row-mt 1'
            self.lbl_info.SetLabel("VP9 - libvpx-vp9")

        self.rdb_deadline.SetSelection(1)
        self.opt["Deadline"] = '-deadline good'
        self.spin_cpu.SetRange(0, 5)
        self.spin_cpu.SetValue(1)
        self.opt["CpuUsed"] = '-cpu-used 1'

        self.opt["Preset"] = ''
        self.opt["Profile"] = ''
        self.opt["Level"] = ''
        self.opt["Tune"] = ''
        self.opt["Usage"] = ''
        self.opt["GOP"] = ''
    # ------------------------------------------------------------------#

    def on_deadline(self, event):
        """
        Sets range according to spin_cpu used
        """
        if self.rdb_deadline.GetSelection() in [0, 1]:
            self.spin_cpu.SetRange(0, 5), self.spin_cpu.SetValue(0)
        else:
            self.spin_cpu.SetRange(0, 15), self.spin_cpu.SetValue(0)
        val = self.rdb_deadline.GetStringSelection()
        self.opt["Deadline"] = f'-deadline {val}'
    # ------------------------------------------------------------------#

    def on_cpu_used(self, event):
        """
        Set cpu-used according to deadline value
        """
        self.opt["CpuUsed"] = f'-cpu-used {self.spin_cpu.GetValue()}'
    # ------------------------------------------------------------------#

    def on_row_mt(self, event):
        """
        Sets row-mt option only for vp9
        """
        if self.ckbx_rowmt.IsChecked():
            self.opt["RowMthreading"] = '-row-mt 1'
        else:
            self.opt["RowMthreading"] = ''



# -*- coding: UTF-8 -*-
"""
Name: two_pass_EBU.py
Porpose: FFmpeg long processing task with EBU normalization
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2023 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Dec.02.2022
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
from threading import Thread
import time
import itertools
import subprocess
import platform
import wx
from pubsub import pub
from videomass.vdms_utils.utils import Popen
from videomass.vdms_io.make_filelog import logwrite
if not platform.system() == 'Windows':
    import shlex


class Loudnorm(Thread):
    """
    Like `TwoPass_Thread` but execute -loudnorm parsing from first
    pass and has definitions to apply on second pass.

    NOTE capturing output in real-time (Windows, Unix):

    https://stackoverflow.com/questions/1388753/how-to-get-output-
    from-subprocess-popen-proc-stdout-readline-blocks-no-dat?rq=1

    """
    get = wx.GetApp()  # get videomass wx.App attribute
    appdata = get.appset
    OS = appdata['ostype']
    NOT_EXIST_MSG = _("Is 'ffmpeg' installed on your system?")

    def __init__(self, logname, duration, timeseq, *args):
        """
        Called from `long_processing_task.topic_thread`.
        Also see `main_frame.switch_to_processing`.
        """
        self.stop_work_thread = False  # process terminate
        self.input_flist = args[1]  # list of infile (elements)
        self.passlist = args[5]  # comand list
        self.audio_outmap = args[6]  # map output list
        self.output_flist = args[3]  # output path
        self.duration = duration  # durations list
        self.time_seq = timeseq  # time segments list
        self.count = 0  # count first for loop
        self.countmax = len(args[1])  # length file list
        self.logname = logname  # title name of file log
        self.nul = 'NUL' if Loudnorm.OS == 'Windows' else '/dev/null'

        Thread.__init__(self)
        self.start()  # start the thread (va in self.run())

    def run(self):
        """
        Subprocess initialize thread.
        """
        filedone = []
        summary = {'Input Integrated:': None, 'Input True Peak:': None,
                   'Input LRA:': None, 'Input Threshold:': None,
                   'Output Integrated:': None, 'Output True Peak:': None,
                   'Output LRA:': None, 'Output Threshold:': None,
                   'Normalization Type:': None, 'Target Offset:': None
                   }
        for (infile,
             outfile,
             duration) in itertools.zip_longest(self.input_flist,
                                                self.output_flist,
                                                self.duration,
                                                fillvalue='',
                                                ):
            # --------------- first pass
            pass1 = (f'"{Loudnorm.appdata["ffmpeg_cmd"]}" -nostdin -loglevel '
                     f'info -stats -hide_banner {self.time_seq} -i "{infile}" '
                     f'{self.passlist[0]} {Loudnorm.appdata["ffthreads"]} -y '
                     f'{self.nul}'
                     )
            self.count += 1
            count = (f'File {self.count}/{self.countmax} - Pass One\n '
                     f'Loudnorm ebu: Getting statistics for measurements...')
            cmd = (f'{count}\nSource: "{infile}"\nDestination: '
                   f'"{self.nul}"\n\n'
                   f'[COMMAND]:\n{pass1}')

            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=count,
                         fsource=f'Source:  "{infile}"',
                         destination=f'Destination: "{self.nul}"',
                         duration=duration,
                         end='',
                         )
            logwrite(cmd, '', self.logname)  # write n/n + command only

            if not Loudnorm.OS == 'Windows':
                pass1 = shlex.split(pass1)
            try:
                with Popen(pass1,
                           stderr=subprocess.PIPE,
                           bufsize=1,
                           universal_newlines=True,
                           encoding='utf8',
                           ) as proc1:

                    for line in proc1.stderr:
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line,
                                     duration=duration,
                                     status=0
                                     )
                        if self.stop_work_thread:  # break first 'for' loop
                            proc1.terminate()
                            break

                        for k in summary:
                            if line.startswith(k):
                                summary[k] = line.split(':')[1].split()[0]

                    if proc1.wait():  # will add '..failed' to txtctrl
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output='',
                                     duration=duration,
                                     status=proc1.wait(),
                                     )
                        logwrite('',
                                 f"Exit status: {proc1.wait()}",
                                 self.logname,
                                 )  # append exit error number
                        break

            except (OSError, FileNotFoundError) as err:
                excepterr = f"{err}\n  {Loudnorm.NOT_EXIST_MSG}"
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count=excepterr,
                             fsource='',
                             destination='',
                             duration=0,
                             end='error'
                             )
                break

            if self.stop_work_thread:  # break first 'for' loop
                proc1.terminate()
                break  # fermo il ciclo for, altrimenti passa avanti

            if proc1.wait() == 0:  # will add '..terminated' to txtctrl
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count='',
                             fsource='',
                             destination='',
                             duration=duration,
                             end='Done'
                             )
            # --------------- second pass ----------------#
            filters = (f'{self.passlist[2]}'
                       f':measured_I={summary["Input Integrated:"]}'
                       f':measured_LRA={summary["Input LRA:"]}'
                       f':measured_TP={summary["Input True Peak:"]}'
                       f':measured_thresh={summary["Input Threshold:"]}'
                       f':offset={summary["Target Offset:"]}'
                       f':linear=true:dual_mono=true'
                       )
            time.sleep(.5)

            pass2 = (f'"{Loudnorm.appdata["ffmpeg_cmd"]}" -nostdin -loglevel '
                     f'info -stats -hide_banner {self.time_seq} -i '
                     f'"{infile}" {self.passlist[1]} '
                     f'-filter:a:{self.audio_outmap[1]} {filters} '
                     f'{Loudnorm.appdata["ffthreads"]} -y "{outfile}"'
                     )
            count = (f'File {self.count}/{self.countmax} - Pass Two\n'
                     f'Loudnorm ebu: apply EBU R128...'
                     )
            cmd = (f'\n{count}\nSource: "{infile}"\n'
                   f'Destination: "{outfile}"\n\n'
                   f'[COMMAND]:\n{pass2}'
                   )

            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=count,
                         fsource=f'Source:  "{infile}"',
                         destination=f'Destination: "{outfile}"',
                         duration=duration,
                         end='',
                         )
            logwrite(cmd, '', self.logname)

            if not Loudnorm.OS == 'Windows':
                pass2 = shlex.split(pass2)
            with Popen(pass2,
                       stderr=subprocess.PIPE,
                       bufsize=1,
                       universal_newlines=True,
                       encoding='utf8',
                       ) as proc2:

                for line2 in proc2.stderr:
                    wx.CallAfter(pub.sendMessage,
                                 "UPDATE_EVT",
                                 output=line2,
                                 duration=duration,
                                 status=0,
                                 )
                    if self.stop_work_thread:  # break first 'for' loop
                        proc2.terminate()
                        break

                if proc2.wait():  # will add '..failed' to txtctrl
                    wx.CallAfter(pub.sendMessage,
                                 "UPDATE_EVT",
                                 output='',
                                 duration=duration,
                                 status=proc2.wait(),
                                 )
                    logwrite('',
                             f"Exit status: {proc2.wait()}",
                             self.logname,
                             )  # append exit error number

            if self.stop_work_thread:  # break first 'for' loop
                proc2.terminate()
                break  # fermo il ciclo for, altrimenti passa avanti

            if proc2.wait() == 0:  # will add '..terminated' to txtctrl
                filedone.append(infile)
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count='',
                             fsource='',
                             destination='',
                             duration=duration,
                             end='Done'
                             )
        time.sleep(.5)
        wx.CallAfter(pub.sendMessage, "END_EVT", msg=filedone)
    # --------------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True



# -*- coding: UTF-8 -*-
"""
Name: settings_manager.py
Porpose: Read and write the configuration file
Compatibility: Python3
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2023 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Jan.11.2023
Code checker: flake8, pylint

 This file is part of Videomass.

    Videomass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Videomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import json


class ConfigManager:
    """
    It represents the setting of the user parameters
    of the program and the configuration file in its
    read and write fondamentals.

    Usage:

    write a new conf.json :
        >>> from settings_manager import ConfigManager
        >>> confmng = ConfigManager('fileconfpath.json')
        >>> confmng.write_options()

    read the current fileconf.json :
        >>> settings = confmng.read_options()

    example of modify data into current file conf.json:
        >>> settings['outputfile'] = '/home/user/MyVideos'
        >>> confmng.write_options(**settings)
    ------------------------------------------------------

    Options description:

    confversion (float):
        current version of this configuration file

    outputfile (str):
        file destination path used by ffmpeg

    outputfile_samedir (bool):
        if True save each ffmpeg files to same folder as source

    filesuffix (str):
        An optional suffix to assign to output files

    ffmpeg_cmd, ffplay_cmd, ffprobe_cmd, (str):
        Absolute or relative path name of the executable.
        If an empty ("") string is found, starts the wizard.

    ffmpeg_islocal, ffplay_islocal, ffprobe_islocal, (bool):
        With True the user enables the executable locally

    ffmpegloglev (str):
        -loglevel one of `error`, `warning`, `info`, `verbose`, `debug`

    ffthreads (str):
        Set the number of threads (from 0 to 32)

    ffplayloglev (str):
        -loglevel one of `quiet`, `fatal`, `error`, `warning`, `info`

    warnexiting (bool):
        with True displays a message dialog before exiting the app

    icontheme (str):
        Icon theme Name (see art folder)

    toolbarsize (int):
        Set toolbar icon size, one of 16, 24, 32, 64 default is 24 px.

    toolbarpos (int):
        Set toolbar positioning. 0 placed on top side;
        1 placed at the bottom side; 2 placed at the rigth side;
        3 is placed at the left side. default is 0 .

    toolbartext (bool):
        True, enables text alongside toolbar buttons. Default is True.

    main_window_size (list):
        [int(Height), int(Width)] last current window dimension before
        exiting the application.

    main_window_pos (list):
        [int(x), in(y)] last current window position on monitor screen
        before exiting the application.

    clearcache (bool):
        with True, clear the cache before exiting the application,
        default is True

    clearlogfiles (bool):
        with True, erases all log files content before exiting the
        application, default is False.

    move_file_to_trash (bool):
        if True, move input files to videomass trash folder or in
        some other dir.
        default value is False

    user_trashdir (str):
        If None, it is set to "conf_trashdir" when the program runs
        (see configurator), user specified Path Name otherwise.

    locale_name (str):
        "Default", set system language to videomass message catalog
        if available, set to English otherwise.
        It supports canonical form of locale names as used on UNIX systems,
        eg. xx or xx_YY format, where xx is ISO 639 code of language and
        YY is ISO 3166 code of the country. Examples are "en", "en_GB",
        "en_US" or "fr_FR", etc.

    dirdownload (str):
        file destination path used by the youtube-dl UI

    use-downloader (bool):
        sets the ability to download videos from YouTube.com.
        One of True or False, where True means load/use yt_dlp
        on sturtup.

    playlistsubfolder (bool):
        Auto-create subfolders when download the playlists,
        default value is True.

    ("ssl_certificate", "add_metadata", "embed_thumbnails",
    "overwr_dl_files", "include_ID_name", "restrict_fname"
    "write_subtitle") (bool)
        Checkboxes option (see YouTube Downloader)

    external_downloader (str):
        external downloader used by yt-dlp. Default is None

    external_downloader_args (list):
        args used by external downloader in yt-dlp. Default is None
        List should be passed using aria2c ["-j", "1", "-x", "1", "-s", "1"]

    """
    VERSION = 6.1
    DEFAULT_OPTIONS = {"confversion": VERSION,
                       "outputfile": f"{os.path.expanduser('~')}",
                       "outputfile_samedir": False,
                       "filesuffix": "",
                       "ffmpeg_cmd": "",
                       "ffmpeg_islocal": False,
                       "ffmpegloglev": "-loglevel warning",
                       "ffthreads": "-threads 4",
                       "ffplay_cmd": "",
                       "ffplay_islocal": False,
                       "ffplayloglev": "-loglevel error",
                       "ffprobe_cmd": "",
                       "ffprobe_islocal": False,
                       "warnexiting": True,
                       "icontheme": "Videomass-Colours",
                       "toolbarsize": 24,
                       "toolbarpos": 0,
                       "toolbartext": True,
                       "main_window_size": [850, 560],
                       "main_window_pos": [0, 0],
                       "main_ytdl_size": [850, 560],
                       "main_ytdl_pos": [0, 0],
                       "clearcache": True,
                       "clearlogfiles": False,
                       "move_file_to_trash": False,
                       "user_trashdir": None,
                       "locale_name": "Default",
                       "dirdownload": f"{os.path.expanduser('~')}",
                       "use-downloader": False,
                       "playlistsubfolder": True,
                       "ssl_certificate": False,
                       "add_metadata": True,
                       "embed_thumbnails": False,
                       "overwr_dl_files": False,
                       "include_ID_name": False,
                       "restrict_fname": False,
                       "write_subtitle": False,
                       "external_downloader": None,
                       "external_downloader_args": None,
                       }

    def __init__(self, filename, makeportable=None):
        """
        Expects an existing `filename` on the file system paths
        suffixed by `.json`. If `makeportable` is `True`, some
        paths on the `DEFAULT_OPTIONS` class attribute will be
        set as relative paths.
        """
        self.filename = filename

        if makeportable:
            path = os.path.join(makeportable, "My_Files")
            outputdir = os.path.relpath(path)
            ConfigManager.DEFAULT_OPTIONS['outputfile'] = outputdir
            ConfigManager.DEFAULT_OPTIONS['dirdownload'] = outputdir

    def write_options(self, **options):
        """
        Writes options to configuration file. If **options is
        given, writes the new changes to filename, writes the
        DEFAULT_OPTIONS otherwise.
        """
        if options:
            set_options = options
        else:
            set_options = ConfigManager.DEFAULT_OPTIONS

        with open(self.filename, "w", encoding='utf-8') as settings_file:

            json.dump(set_options,
                      settings_file,
                      indent=4,
                      separators=(",", ": ")
                      )

    def read_options(self):
        """
        Reads options from the current configuration file.
        Returns: current options, `None` otherwise.
        Raise: json.JSONDecodeError
        """
        with open(self.filename, 'r', encoding='utf-8') as settings_file:
            try:
                options = json.load(settings_file)
            except json.JSONDecodeError:
                return None

        return options
